@@ -263,14 +263,15 @@ class Composite(Datum):
     topology: Dict[str, Any] = {}
     _schema: Dict[str, Any] = {}
     defaults: Dict[str, Any] = {
-        'processes': processes,
-        'topology': topology,
-        '_schema': _schema}
+        'processes': dict,
+        'topology': dict,
+        '_schema': dict}
 
     def __init__(
             self,
-            config: Dict[str, Any]
+            config: Optional[Dict[str, Any]] = None
     ) -> None:
+        config = config or {}
         super().__init__(config)
         _override_schemas(self._schema, self.processes)
 
@@ -490,7 +491,16 @@ class MetaComposer(Composer):
             self,
             composers: Iterable[Any] = tuple(),
             config: Optional[dict] = None,
-    ) -> None:
+            ) -> None:
+        """A collection of :py:class:`Composer` objects.
+
+        The MetaComposer can be used to create composites that combine
+        all the composers in the collection.
+
+        Args:
+            composers: Initial collection of composers.
+            config: Initial configuration.
+        """
         super().__init__(config)
         self.composers: List = list(composers)
 
@@ -498,6 +508,7 @@ class MetaComposer(Composer):
             self,
             config: Optional[dict] = None
     ) -> Dict[str, Any]:
+        """Do not override this method."""
         # TODO(Eran)-- override composite.config with config
         processes: Dict = {}
         for composer in self.composers:
@@ -513,6 +524,7 @@ class MetaComposer(Composer):
             self,
             config: Optional[dict] = None
     ) -> Topology:
+        """Do not override this method."""
         topology: Topology = {}
         for composer in self.composers:
             new_topology = composer.generate_topology(composer.config)
@@ -528,6 +540,13 @@ class MetaComposer(Composer):
             composer: Composer,
             config: Optional[Dict] = None,
     ) -> None:
+        """Add a composer to the collection of stored composers.
+
+        Args:
+            composer: The composer to add.
+            config: The composer's configuration, which will be merged
+                with the stored config.
+        """
         if config:
             self.config.update(config)
         self.composers.append(composer)
@@ -537,6 +556,13 @@ class MetaComposer(Composer):
             composers: List,
             config: Optional[Dict] = None,
     ) -> None:
+        """Add multiple composers to the collection of stored composers.
+
+        Args:
+            composers: The composers to add.
+            config: Configuration for the composers, which will be
+                merged with the stored config.
+        """
         if config:
             self.config.update(config)
         self.composers.extend(composers)
@@ -582,11 +608,13 @@ class Process(Composer, metaclass=abc.ABCMeta):
 
     def generate_processes(
             self, config: Optional[dict] = None) -> Dict[str, Any]:
+        """Do not override this method."""
         config = config or {}
         name = config.get('name', self.name)
         return {name: self}
 
     def generate_topology(self, config: Optional[dict] = None) -> Topology:
+        """Do not override this method."""
         config = config or {}
         name = config.get('name', self.name)
         override_topology = config.get('topology', {})
@@ -731,6 +759,11 @@ class Process(Composer, metaclass=abc.ABCMeta):
 class Deriver(Process, metaclass=abc.ABCMeta):
     """Base class for :term:`derivers`."""
     def initial_state(self, config: Optional[dict] = None) -> State:
+        """Subclasses should override this method.
+
+        Given a config, this method should return the Deriver's initial
+        state.
+        """
         return {}
 
     def is_deriver(self) -> bool:
